/*  
 *  gpca/gdl_gpca_result.h 
 *  
 *  $Author: baptiste $, $Date: 2008-05-13 15:33:42 $, $Version$
 *
 *  Libgdl : a C library for statistical genetics
 * 
 *  Copyright (C) 2003-2006  Jean-Baptiste Veyrieras, INRA, France.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA * 
 */

#ifndef __GDL_GPCA_RESULT_H__
#define __GDL_GPCA_RESULT_H_

#include <gdl/gdl_common.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_odb.h>
#include <gdl/gdl_string.h>
#include <gdl/gdl_matrix.h>
#include <gdl/gdl_vector.h>
#include <gdl/gdl_pca.h>
#include <gdl/gdl_plot.h>
#include <gdl/gdl_gpca.h>

__BEGIN_DECLS

typedef struct _gdl_gpca_result gdl_gpca_result;

gdl_gpca_result * gdl_gpca_result_alloc (const gdl_gpca_workspace * w);
void gdl_gpca_result_free (gdl_gpca_result * r);

size_t gdl_gpca_result_naxe (const gdl_gpca_result * r);
gdl_matrix * gdl_gpca_result_projection (const gdl_gpca_result * r);
gdl_matrix * gdl_gpca_result_rotation (const gdl_gpca_result * r);
gdl_vector * gdl_gpca_result_weights (const gdl_gpca_result * r);
gdl_vector * gdl_gpca_result_tracy_widom (const gdl_gpca_result * r);

const gdl_gpca_workspace_type * gdl_gpca_result_get_type (const gdl_gpca_result * r);
const gdl_glabels * gdl_gpca_result_get_labels (const gdl_gpca_result * r);
gdl_string * gdl_gpca_result_get_row_label (const gdl_gpca_result * r, size_t i);
gdl_string * gdl_gpca_result_get_column_label (const gdl_gpca_result * r, size_t i);

gdl_gpca_result * gdl_gpca_result_fread (FILE * stream);
int gdl_gpca_result_fwrite (FILE * stream, const gdl_gpca_result * r);
int gdl_gpca_result_fprintf (FILE * stream, const gdl_gpca_result * r);

GDL_VAR const gdl_plot_type * gdl_gpca_result_plot;
GDL_VAR const gdl_odriver * gdl_gpca_result_driver;

__END_DECLS

#endif
