/*  
 *  hsmap/part.c
 * 
 *  $Author: baptiste $, $Date: 2008-05-13 15:33:44 $, $Version$
 *
 *  Libgdl : a C library for statistical genetics
 * 
 *  Copyright (C) 2003-2006  Jean-Baptiste Veyrieras, INRA, France.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA * 
 */

#include <gdl/gdl_common.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_rng.h>
#include <gdl/gdl_gentity.h>
#include <gdl/gdl_mask.h>
#include <gdl/gdl_gview.h>
#include <gdl/gdl_gview_wrapper.h>
#include <gdl/gdl_gmap.h>
#include <gdl/gdl_hview.h>
#include <gdl/gdl_hmap.h>
#include <gdl/gdl_fview_wrapper.h>
#include <gdl/gdl_view.h>
#include <gdl/gdl_hstruct_model.h>
#include <gdl/gdl_hstruct_result.h>
#include <gdl/gdl_hsmap_partition.h>

gdl_hsmap_partition *
gdl_hsmap_partition_alloc (const gdl_hstruct_partition_result * result, const gdl_hstruct_model_partition * model)
{
	gdl_hsmap_partition * part;
	
	part = GDL_CALLOC (gdl_hsmap_partition, 1);

	part->result = result;	
	part->model  = model;
	part->data   = gdl_hstruct_model_partition_get_wrapper (model);
	part->matrix = gdl_gmatrix_wrapper_alloc (gdl_gmatrix_allele, part->data, gdl_false, gdl_false);
	part->mask   = gdl_mask_alloc ();
	
	return part;
}

void
gdl_hsmap_partition_free (gdl_hsmap_partition * p)
{
	if (p)
	{
		gdl_gview_wrapper_free (p->data);
		gdl_mask_free (p->mask);
		gdl_mask_free (p->tmask);
		GDL_FREE (p);
	}	
}


