/* statistics/gdl_statistics_char.h
 * 
 * Copyright (C) 1996, 1997, 1998, 1999, 2000 Jim Davies, Brian Gough
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __GDL_STATISTICS_CHAR_H__
#define __GDL_STATISTICS_CHAR_H__

#include <stddef.h>

#include <gdl/gdl_common.h>

__BEGIN_DECLS

double gdl_stats_char_mean (const char data[], const size_t stride, const size_t n);
double gdl_stats_char_variance (const char data[], const size_t stride, const size_t n);
double gdl_stats_char_sd (const char data[], const size_t stride, const size_t n);
double gdl_stats_char_variance_with_fixed_mean (const char data[], const size_t stride, const size_t n, const double mean);
double gdl_stats_char_sd_with_fixed_mean (const char data[], const size_t stride, const size_t n, const double mean);
double gdl_stats_char_absdev (const char data[], const size_t stride, const size_t n);
double gdl_stats_char_skew (const char data[], const size_t stride, const size_t n);
double gdl_stats_char_kurtosis (const char data[], const size_t stride, const size_t n);
double gdl_stats_char_lag1_autocorrelation (const char data[], const size_t stride, const size_t n);

double gdl_stats_char_covariance (const char data1[], const size_t stride1,const char data2[], const size_t stride2, const size_t n);

double gdl_stats_char_variance_m (const char data[], const size_t stride, const size_t n, const double mean);
double gdl_stats_char_sd_m (const char data[], const size_t stride, const size_t n, const double mean);
double gdl_stats_char_absdev_m (const char data[], const size_t stride, const size_t n, const double mean);
double gdl_stats_char_skew_m_sd (const char data[], const size_t stride, const size_t n, const double mean, const double sd);
double gdl_stats_char_kurtosis_m_sd (const char data[], const size_t stride, const size_t n, const double mean, const double sd);
double gdl_stats_char_lag1_autocorrelation_m (const char data[], const size_t stride, const size_t n, const double mean);

double gdl_stats_char_covariance_m (const char data1[], const size_t stride1,const char data2[], const size_t stride2, const size_t n, const double mean1, const double mean2);


double gdl_stats_char_pvariance (const char data1[], const size_t stride1, const size_t n1, const char data2[], const size_t stride2, const size_t n2);
double gdl_stats_char_ttest (const char data1[], const size_t stride1, const size_t n1, const char data2[], const size_t stride2, const size_t n2);

char gdl_stats_char_max (const char data[], const size_t stride, const size_t n);
char gdl_stats_char_min (const char data[], const size_t stride, const size_t n);
void gdl_stats_char_minmax (char * min, char * max, const char data[], const size_t stride, const size_t n);

size_t gdl_stats_char_max_index (const char data[], const size_t stride, const size_t n);
size_t gdl_stats_char_min_index (const char data[], const size_t stride, const size_t n);
void gdl_stats_char_minmax_index (size_t * min_index, size_t * max_index, const char data[], const size_t stride, const size_t n);

double gdl_stats_char_median_from_sorted_data (const char sorted_data[], const size_t stride, const size_t n) ;
double gdl_stats_char_quantile_from_sorted_data (const char sorted_data[], const size_t stride, const size_t n, const double f) ;

__END_DECLS

#endif /* __GDL_STATISTICS_CHAR_H__ */
